<?xml version="1.0" encoding="ISO-8859-1"?>

<!-- XSLT stylesheet template fragment:
	Draws data circles for a pie chart.
-->
<!-- $Id$ -->

<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:java="http://xml.apache.org/xslt/java"
  exclude-result-prefixes="java"
  version="1.0"
  >

  <!-- In a pie chart, each column of data is plotted as a separate pie,
	with the same centers and progressively smaller radii.  Each
	slice of the pie represents a value from one row of the column
	used for that pie.  Thus for pie charts, each row is assigned
	a different color instead of each column so that each slice has
	a different color.
	We use a recursive template to iterate over each column to plot
	each pie, and iterate the rows within that column to plot each
	slice.
    -->

  <!-- This template matches the data of the graph.  It creates a coordinate
	system for the pie chart, and calls the recursive template to
	create each pie.
    -->
  <xsl:template match="data">
    <!-- Establish the coordinate system at the center of the plot area. -->
    <xsl:text>    </xsl:text>
    <svg x="0" y="0" width="100" height="100" preserveAspectRatio="xMidYMid"
	stroke="black" stroke-width="0.2">
      <xsl:choose>
	<xsl:when test="/graph/@style = 'exploded' or /graph/@style = 'extract'">
	<xsl:attribute name="viewBox">-106 -106 212 212</xsl:attribute>
	</xsl:when>
	<xsl:otherwise>
	<xsl:attribute name="viewBox">-100 -100 200 200</xsl:attribute>
	</xsl:otherwise>
      </xsl:choose>
      <xsl:text>&#10;</xsl:text>
      <xsl:call-template name="pie"/>
      <xsl:text>    </xsl:text>
    </svg>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

  <!-- This is a recursive template that iterates over each column of
	data to plot each pie.  For each pie, it determines its radius
	and iterates over each row within the column to plot each slice.
    -->
  <xsl:template name="pie">
    <xsl:param name="col" select="1"/>
    <xsl:variable name="cols" select="number(@cols)"/>
    <!-- This is the recursion termination test. -->
    <xsl:if test="$cols >= $col">
      <xsl:variable name="radius" select="98 - (($col - 1) div $cols) * 100"/>
      <xsl:variable name="total" select="sum(row/col[$col])"/>
      <!-- Only plot the pie if there is some data there to plot. -->
      <xsl:if test="$total > 0">
	<!-- Iterate over each row to plot each slice. -->
	<xsl:for-each select="row/col[$col]">
	  <xsl:call-template name="slice">
	    <xsl:with-param name="col" select="$col"/>
	    <xsl:with-param name="radius" select="$radius"/>
	    <xsl:with-param name="total" select="$total"/>
	  </xsl:call-template>
	</xsl:for-each>
      </xsl:if>
      <!-- Recurse. -->
      <xsl:call-template name="pie">
	<xsl:with-param name="col" select="$col + 1"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <!-- This plots each slice of the pie.  It determines the angle for
	the slice, and how far to rotate it around the pie (based on
	the sum of all the previous angles).  It then produces the
	filled path for the slice.
	When we draw each slice, we create a path starting on the edge
	of the pie, arcing around the outside edge of the slice, straight
	in to the center, then straight out to the starting point.  The
	SVG arc function uses two flags to determine where to center the
	arc: the large-arc-flag and the sweep-flag.  If the slice is less
	than half of the pie, the large-arc-flag is zero, otherwise it is
	set to one.  This ensures that for large slices we use the larger
	arc.  We always set the sweep-flag to one in order to draw the
	arc clockwise from our starting position.
    -->
  <xsl:template name="slice">
    <xsl:param name="col"/>
    <xsl:param name="radius"/>
    <xsl:param name="total"/>
    <xsl:variable name="this" select="position()"/>
    <!-- The strangeness of sum(.) below handles the case of non-numeric
	data, because sum(.) will return zero, while just . may be NaN.
      -->
    <xsl:variable name="fraction" select="sum(.) div $total"/>
    <xsl:variable name="angle"
		select="java:java.lang.Math.toRadians($fraction * 360.0)"/>
    <xsl:variable name="sine" select="java:java.lang.Math.sin($angle)"/>
    <xsl:variable name="cosine" select="java:java.lang.Math.cos($angle)"/>
    <xsl:variable name="before"
		select="sum(../../row[position()&lt;$this]/col[$col])"/>
    <xsl:text>      </xsl:text>
    <g>
    <xsl:text>&#10;        </xsl:text>
    <title><xsl:value-of select="../@name"/></title>
    <xsl:text>&#10;        </xsl:text>
    <xsl:choose>
      <!-- If this slice accounts for the entire total of the pie, then
	   we just draw a circle.  This can happen when there is just
	   one row, or one row containing numeric data, in this column.
	-->
      <xsl:when test="sum(.) = $total">
	<circle r="{$radius}" fill="{@color}"/>
      </xsl:when>
      <!-- For slices that should be extracted from the center, shift
	   the slice out along a line that bisects the angle of the
	   slice.  We shift a constant number of units (6).
	-->
      <xsl:when test="/graph/@style = 'exploded' or
		(/graph/@style = 'extract' and position() = 1)">
	<path transform="rotate({($before div $total) * 360 - 90})
		 translate({java:java.lang.Math.cos($angle div 2) * 6},
			   {java:java.lang.Math.sin($angle div 2) * 6})"
		fill="{@color}"
		d="M {$radius},0 A {$radius} {$radius} 0
			{number($fraction > 0.5)} 1
			{$cosine * $radius} {$sine * $radius} L 0 0 Z"/>
      </xsl:when>
      <!-- For normal slices, we draw a line from the origin to the
	   radius, arc over the angle represented by this slice, then
	   close and fill it.
	-->
      <xsl:otherwise>
	<path transform="rotate({($before div $total) * 360 - 90})"
		fill="{@color}"
		d="M {$radius},0 A {$radius} {$radius} 0
			{number($fraction > 0.5)} 1
			{$cosine * $radius} {$sine * $radius} L 0 0 Z"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>&#10;      </xsl:text>
    </g>
    <xsl:text>&#10;</xsl:text>
  </xsl:template>

</xsl:stylesheet>
